import sys
sys.path.append('.')
import lvgl as lv
import lv_utils
import time
import uctypes
import machine

# DMA Driver
class DMA(object):
    DMA_BASE  = const(0x50000000)

    DMA_EN    = const(0x01 << 0)
    HIGH_PRIO = const(0x01 << 1)
    INCR_READ = const(0x01 << 4)
    INCR_WRITE= const(0x01 << 5)
    DREQ_PIO0_RX0 = const(0x04 << 15)
    DREQ_SPI1_TX  = const(0x12 << 15)
    DREQ_PERMANENT= const(0x3F << 15)
    IRQ_QUIET = const(0x01 << 21)
    BUSY      = const(0x01 << 24)

    def __init__( self, channelNumber ):
        offset = channelNumber * 0x40
        self.CHx_READ_ADDR     = DMA.DMA_BASE + 0x00 + offset
        self.CHx_WRITE_ADDR    = DMA.DMA_BASE + 0x04 + offset
        self.CHx_TRANS_COUNT   = DMA.DMA_BASE + 0x08 + offset
        self.CHx_CTRL_TRIG     = DMA.DMA_BASE + 0x0C + offset

    def config( self, src_addr, dst_addr, count, src_inc, dst_inc, trig_dreq ):
        machine.mem32[ self.CHx_CTRL_TRIG ]   = 0
        machine.mem32[ self.CHx_READ_ADDR ]   = src_addr
        machine.mem32[ self.CHx_WRITE_ADDR ]  = dst_addr
        machine.mem32[ self.CHx_TRANS_COUNT ] = count
        trig_val = 0
        if( src_inc ):
            trig_val |= DMA.INCR_READ
        if( dst_inc ):
            trig_val |= DMA.INCR_WRITE
        trig_val |= trig_dreq
        machine.mem32[ self.CHx_CTRL_TRIG ]   = trig_val

    def enable( self ):
        machine.mem32[ self.CHx_CTRL_TRIG ] |= DMA.DMA_EN

    def disable( self ):
        machine.mem32[ self.CHx_CTRL_TRIG ] = 0

    def is_busy( self ):
        if( machine.mem32[ self.CHx_CTRL_TRIG ] & DMA.BUSY ):
            return True
        else:
            return False

class LVGL(object):
    def rp2_wait_dma(self):
        while self.rp2_dma.is_busy(): pass
        self.rp2_dma.disable()
        # wait to send last byte. It should take < 1uS @ 10MHz
        time.sleep_us(1)
        self.LCD.cs.value(1)
        
    def disp_drv_flush_cb(self,disp_drv,area,color_p):
        self.rp2_wait_dma() # wait if not yet done and DMA is being used
        self.buf = bytearray(1)
        
        w = area.x2 - area.x1 + 1
        h = area.y2 - area.y1 + 1
        size = w * h
        data_view = color_p.__dereference__(size * self.pixel_size)
        self.rgb565_swap_func(data_view, size)
        # Set windows
        self.LCD.setWindows(area.x1, area.y1, area.x2+1, area.y2+1)
    
        SPI1_BASE = 0x40040000 # FIXME: will be different for another SPI bus?
        SSPDR     = 0x008
        self.rp2_dma.config(
            src_addr = uctypes.addressof(data_view),
            dst_addr = SPI1_BASE + SSPDR,
            count    = len(data_view),
            src_inc  = True,
            dst_inc  = False,
            trig_dreq= self.rp2_dma.DREQ_SPI1_TX
        )

        RAM_WR  = 0x2C
        self.LCD.write_cmd(RAM_WR)
        self.LCD.cs.value(0)
        self.LCD.dc.value(1)
    
        self.rp2_dma.enable()

        self.rp2_wait_dma()
               
        self.disp_drv.flush_ready()
        
    def indev_drv_read_cb( self, indev_drv, data):
        self.rp2_wait_dma()  

        data.point.x = self.TSC.X_point
        data.point.y = self.TSC.Y_point
        
        data.state = 1 if self.TSC.Flag == 1 else 0 
        self.TSC.Flag = 0
    
    def __init__(self,LCD=None,TSC=None):
        # Init parameter
        self.LCD = LCD
        self.TSC = TSC
        self.is_buf1 = True
        
        # Set color format
        color_format = lv.COLOR_FORMAT.RGB565
        self.pixel_size = lv.color_format_get_size(color_format)
        self.rgb565_swap_func = lv.draw_sw_rgb565_swap

        # Init LVGL Core
        if not lv.is_initialized(): lv.init()

        # Create event loop if not yet present
        if not lv_utils.event_loop.is_running(): self.event_loop=lv_utils.event_loop()

        # Init LVGL display
        self.buf1 = lv.draw_buf_create(self.LCD.width, self.LCD.height // 3 , color_format, 0)
        self.buf2 = lv.draw_buf_create(self.LCD.width, self.LCD.height // 3, color_format, 0)   
        self.disp_drv = lv.display_create(self.LCD.width, self.LCD.height)
        self.disp_drv.set_color_format(color_format)
        self.disp_drv.set_draw_buffers(self.buf1, self.buf2)
        self.disp_drv.set_render_mode(lv.DISPLAY_RENDER_MODE.PARTIAL)
        self.disp_drv.set_flush_cb(self.disp_drv_flush_cb)
        
        # Init touch screen as input device
        self.indev_drv = lv.indev_create()
        self.indev_drv.set_type(lv.INDEV_TYPE.POINTER)
        self.indev_drv.set_read_cb(self.indev_drv_read_cb)
        
        # Init DMA
        self.rp2_dma = DMA(0)


